@@ -13,7 +13,9 @@ futureAssignInternal <- function(target, expr, envir=parent.frame(), substitute=
       } else if (all(is.finite(target$idx))) {
         name <- get_variable(target$envir, target$idx, mustExist=FALSE, create=TRUE)
       } else {
-        stop("INTERNAL ERROR: Zero length variable name and unknown index.")
+        msg <- "INTERNAL ERROR: Zero length variable name and unknown index."
+        mdebug(msg)
+        stop(msg)
       }
     }
   }
