@@ -6,9 +6,8 @@ Backported for python-future from Python 3.3 test/support.py.
 
 from __future__ import (absolute_import, division,
                         print_function, unicode_literals)
-import future.standard_library
-from future import six
-from future import *
+from future import utils
+from future.builtins import str, range, open, int, map, list
 
 
 # if __name__ != 'test.support':
@@ -25,21 +24,31 @@ import platform
 import shutil
 import warnings
 import unittest
+# For Python 2.6 compatibility:
+if not hasattr(unittest, 'skip'):
+    import unittest2 as unittest
+
 import importlib
 # import collections.abc    # not present on Py2.7
 import re
 import subprocess
 import imp
 import time
-import sysconfig
+try:
+    import sysconfig
+except ImportError:
+    # sysconfig is not available on Python 2.6. Try using distutils.sysconfig instead:
+    from distutils import sysconfig
 import fnmatch
 import logging.handlers
 import struct
 import tempfile
-import _testcapi
 
 try:
-    import _thread, threading
+    if utils.PY3:
+        import _thread, threading
+    else:
+        import thread as _thread, threading
 except ImportError:
     _thread = None
     threading = None
@@ -53,6 +62,11 @@ try:
 except ImportError:
     zlib = None
 
+try:
+    import gzip
+except ImportError:
+    gzip = None
+
 try:
     import bz2
 except ImportError:
@@ -82,7 +96,7 @@ __all__ = [
     "TestHandler", "Matcher", "can_symlink", "skip_unless_symlink",
     "skip_unless_xattr", "import_fresh_module", "requires_zlib",
     "PIPE_MAX_SIZE", "failfast", "anticipate_failure", "run_with_tz",
-    "requires_bz2", "requires_lzma", "suppress_crash_popup",
+    "requires_gzip", "requires_bz2", "requires_lzma", "suppress_crash_popup",
     ]
 
 class Error(Exception):
@@ -130,7 +144,8 @@ def import_module(name, deprecated=False):
 def _save_and_remove_module(name, orig_modules):
     """Helper function to save and remove a module from sys.modules
 
-       Raise ImportError if the module can't be imported."""
+    Raise ImportError if the module can't be imported.
+    """
     # try to import the module and raise an error if it can't be imported
     if name not in sys.modules:
         __import__(name)
@@ -143,7 +158,8 @@ def _save_and_remove_module(name, orig_modules):
 def _save_and_block_module(name, orig_modules):
     """Helper function to save and block a module in sys.modules
 
-       Return True if the module was in sys.modules, False otherwise."""
+    Return True if the module was in sys.modules, False otherwise.
+    """
     saved = True
     try:
         orig_modules[name] = sys.modules[name]
@@ -165,18 +181,32 @@ def anticipate_failure(condition):
 
 
 def import_fresh_module(name, fresh=(), blocked=(), deprecated=False):
-    """Imports and returns a module, deliberately bypassing the sys.modules cache
-    and importing a fresh copy of the module. Once the import is complete,
-    the sys.modules cache is restored to its original state.
+    """Import and return a module, deliberately bypassing sys.modules.
+    This function imports and returns a fresh copy of the named Python module
+    by removing the named module from sys.modules before doing the import.
+    Note that unlike reload, the original module is not affected by
+    this operation.
 
-    Modules named in fresh are also imported anew if needed by the import.
-    If one of these modules can't be imported, None is returned.
+    *fresh* is an iterable of additional module names that are also removed
+    from the sys.modules cache before doing the import.
 
-    Importing of modules named in blocked is prevented while the fresh import
-    takes place.
+    *blocked* is an iterable of module names that are replaced with None
+    in the module cache during the import to ensure that attempts to import
+    them raise ImportError.
+
+    The named module and any modules named in the *fresh* and *blocked*
+    parameters are saved before starting the import and then reinserted into
+    sys.modules when the fresh import is complete.
+
+    Module and package deprecation messages are suppressed during this import
+    if *deprecated* is True.
+
+    This function will raise ImportError if the named module cannot be
+    imported.
 
     If deprecated is True, any module or package deprecation messages
-    will be suppressed."""
+    will be suppressed.
+    """
     # NOTE: test_heapq, test_json and test_warnings include extra sanity checks
     # to make sure that this utility function is working as expected
     with _ignore_deprecated_imports(deprecated):
@@ -238,7 +268,7 @@ def unload(name):
 
 if sys.platform.startswith("win"):
     def _waitfor(func, pathname, waitall=False):
-        # Peform the operation
+        # Perform the operation
         func(pathname)
         # Now setup the wait loop
         if waitall:
@@ -254,7 +284,7 @@ if sys.platform.startswith("win"):
         # required when contention occurs.
         timeout = 0.001
         while timeout < 1.0:
-            # Note we are only testing for the existance of the file(s) in
+            # Note we are only testing for the existence of the file(s) in
             # the contents of the directory regardless of any security or
             # access rights.  If we have made it this far, we have sufficient
             # permissions to do that much using Python's equivalent of the
@@ -422,6 +452,8 @@ def _requires_unix_version(sysname, min_version):
                         raise unittest.SkipTest(
                             "%s version %s or higher required, not %s"
                             % (sysname, min_version_txt, version_txt))
+            return func(*args, **kw)
+        wrapper.min_version = min_version
         return wrapper
     return decorator
 
@@ -470,8 +502,11 @@ def requires_mac_ver(*min_version):
         return wrapper
     return decorator
 
+# Don't use "localhost", since resolving it uses the DNS under recent
+# Windows versions (see issue #18792).
+HOST = "127.0.0.1"
+HOSTv6 = "::1"
 
-HOST = 'localhost'
 
 def find_unused_port(family=socket.AF_INET, socktype=socket.SOCK_STREAM):
     """Returns an unused port that should be suitable for binding.  This is
@@ -556,9 +591,15 @@ def bind_port(sock, host=HOST):
                 raise TestFailed("tests should never set the SO_REUSEADDR "   \
                                  "socket option on TCP/IP sockets!")
         if hasattr(socket, 'SO_REUSEPORT'):
-            if sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT) == 1:
-                raise TestFailed("tests should never set the SO_REUSEPORT "   \
-                                 "socket option on TCP/IP sockets!")
+            try:
+                if sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT) == 1:
+                    raise TestFailed("tests should never set the SO_REUSEPORT "   \
+                                     "socket option on TCP/IP sockets!")
+            except OSError:
+                # Python's socket module was compiled using modern headers
+                # thus defining SO_REUSEPORT but this process is running
+                # under an older kernel that does not support SO_REUSEPORT.
+                pass
         if hasattr(socket, 'SO_EXCLUSIVEADDRUSE'):
             sock.setsockopt(socket.SOL_SOCKET, socket.SO_EXCLUSIVEADDRUSE, 1)
 
@@ -584,11 +625,19 @@ def _is_ipv6_enabled():
 IPV6_ENABLED = _is_ipv6_enabled()
 
 
-# A constant likely larger than the underlying OS pipe buffer size.
-# Windows limit seems to be around 512B, and many Unix kernels have a 64K pipe
-# buffer size or 16*PAGE_SIZE: take a few megs to be sure.  This
-PIPE_MAX_SIZE = 3 * 1000 * 1000
+# A constant likely larger than the underlying OS pipe buffer size, to
+# make writes blocking.
+# Windows limit seems to be around 512 B, and many Unix kernels have a
+# 64 KiB pipe buffer size or 16 * PAGE_SIZE: take a few megs to be sure.
+# (see issue #17835 for a discussion of this number).
+PIPE_MAX_SIZE = 4 * 1024 * 1024 + 1
 
+# A constant likely larger than the underlying OS socket buffer size, to make
+# writes blocking.
+# The socket buffer sizes can usually be tuned system-wide (e.g. through sysctl
+# on Linux), or on a per-socket basis (SO_SNDBUF/SO_RCVBUF). See issue #18643
+# for a discussion of this number).
+SOCK_MAX_SIZE = 16 * 1024 * 1024 + 1
 
 # # decorator for skipping tests on non-IEEE 754 platforms
 # requires_IEEE_754 = unittest.skipUnless(
@@ -612,7 +661,7 @@ else:
 
 # Disambiguate TESTFN for parallel testing, while letting it remain a valid
 # module name.
-TESTFN = "{}_{}_tmp".format(TESTFN, os.getpid())
+TESTFN = "{0}_{1}_tmp".format(TESTFN, os.getpid())
 
 # # FS_NONASCII: non-ASCII character encodable by os.fsencode(),
 # # or None if there is no such character.
@@ -834,11 +883,12 @@ def check_syntax_error(testcase, statement):
                           '<test string>', 'exec')
 
 def open_urlresource(url, *args, **kw):
-    import urllib.request, urllib.parse
+    from future.backports.urllib import (request as urllib_request,
+                                         parse as urllib_parse)
 
     check = kw.pop('check', None)
 
-    filename = urllib.parse.urlparse(url)[2].split('/')[-1] # '/': it's URL!
+    filename = urllib_parse.urlparse(url)[2].split('/')[-1] # '/': it's URL!
 
     fn = os.path.join(os.path.dirname(__file__), "data", filename)
 
@@ -861,7 +911,7 @@ def open_urlresource(url, *args, **kw):
     requires('urlfetch')
 
     print('\tfetching %s ...' % url, file=get_original_stdout())
-    f = urllib.request.urlopen(url, timeout=15)
+    f = urllib_request.urlopen(url, timeout=15)
     try:
         with open(fn, "wb") as out:
             s = f.read()
@@ -910,7 +960,10 @@ def _filterwarnings(filters, quiet=False):
     frame = sys._getframe(2)
     registry = frame.f_globals.get('__warningregistry__')
     if registry:
-        registry.clear()
+        # Was: registry.clear()
+        # Py2-compatible:
+        for i in range(len(registry)):
+            registry.pop()
     with warnings.catch_warnings(record=True) as w:
         # Set filter "always" to record all warnings.  Because
         # test_warnings swap the module, we need to look up in
@@ -995,7 +1048,7 @@ class CleanImport(object):
         sys.modules.update(self.original_modules)
 
 ### Added for python-future:
-if six.PY3:
+if utils.PY3:
     import collections.abc
     mybase = collections.abc.MutableMapping
 else:
@@ -1114,69 +1167,73 @@ socket_peer_reset = TransientResource(socket.error, errno=errno.ECONNRESET)
 ioerror_peer_reset = TransientResource(IOError, errno=errno.ECONNRESET)
 
 
-# @contextlib.contextmanager
-# def transient_internet(resource_name, timeout=30.0, errnos=()):
-#     """Return a context manager that raises ResourceDenied when various issues
-#     with the Internet connection manifest themselves as exceptions."""
-#     default_errnos = [
-#         ('ECONNREFUSED', 111),
-#         ('ECONNRESET', 104),
-#         ('EHOSTUNREACH', 113),
-#         ('ENETUNREACH', 101),
-#         ('ETIMEDOUT', 110),
-#     ]
-#     default_gai_errnos = [
-#         ('EAI_AGAIN', -3),
-#         ('EAI_FAIL', -4),
-#         ('EAI_NONAME', -2),
-#         ('EAI_NODATA', -5),
-#         # Encountered when trying to resolve IPv6-only hostnames
-#         ('WSANO_DATA', 11004),
-#     ]
-# 
-#     denied = ResourceDenied("Resource %r is not available" % resource_name)
-#     captured_errnos = errnos
-#     gai_errnos = []
-#     if not captured_errnos:
-#         captured_errnos = [getattr(errno, name, num)
-#                            for (name, num) in default_errnos]
-#         gai_errnos = [getattr(socket, name, num)
-#                       for (name, num) in default_gai_errnos]
-# 
-#     def filter_error(err):
-#         n = getattr(err, 'errno', None)
-#         if (isinstance(err, socket.timeout) or
-#             (isinstance(err, socket.gaierror) and n in gai_errnos) or
-#             n in captured_errnos):
-#             if not verbose:
-#                 sys.stderr.write(denied.args[0] + "\n")
-#             raise denied from err
-# 
-#     old_timeout = socket.getdefaulttimeout()
-#     try:
-#         if timeout is not None:
-#             socket.setdefaulttimeout(timeout)
-#         yield
-#     except IOError as err:
-#         # urllib can wrap original socket errors multiple times (!), we must
-#         # unwrap to get at the original error.
-#         while True:
-#             a = err.args
-#             if len(a) >= 1 and isinstance(a[0], IOError):
-#                 err = a[0]
-#             # The error can also be wrapped as args[1]:
-#             #    except socket.error as msg:
-#             #        raise IOError('socket error', msg).with_traceback(sys.exc_info()[2])
-#             elif len(a) >= 2 and isinstance(a[1], IOError):
-#                 err = a[1]
-#             else:
-#                 break
-#         filter_error(err)
-#         raise
-#     # XXX should we catch generic exceptions and look for their
-#     # __cause__ or __context__?
-#     finally:
-#         socket.setdefaulttimeout(old_timeout)
+@contextlib.contextmanager
+def transient_internet(resource_name, timeout=30.0, errnos=()):
+    """Return a context manager that raises ResourceDenied when various issues
+    with the Internet connection manifest themselves as exceptions."""
+    default_errnos = [
+        ('ECONNREFUSED', 111),
+        ('ECONNRESET', 104),
+        ('EHOSTUNREACH', 113),
+        ('ENETUNREACH', 101),
+        ('ETIMEDOUT', 110),
+    ]
+    default_gai_errnos = [
+        ('EAI_AGAIN', -3),
+        ('EAI_FAIL', -4),
+        ('EAI_NONAME', -2),
+        ('EAI_NODATA', -5),
+        # Encountered when trying to resolve IPv6-only hostnames
+        ('WSANO_DATA', 11004),
+    ]
+
+    denied = ResourceDenied("Resource %r is not available" % resource_name)
+    captured_errnos = errnos
+    gai_errnos = []
+    if not captured_errnos:
+        captured_errnos = [getattr(errno, name, num)
+                           for (name, num) in default_errnos]
+        gai_errnos = [getattr(socket, name, num)
+                      for (name, num) in default_gai_errnos]
+
+    def filter_error(err):
+        n = getattr(err, 'errno', None)
+        if (isinstance(err, socket.timeout) or
+            (isinstance(err, socket.gaierror) and n in gai_errnos) or
+            n in captured_errnos):
+            if not verbose:
+                sys.stderr.write(denied.args[0] + "\n")
+            # Was: raise denied from err
+            # For Python-Future:
+            exc = denied
+            exc.__cause__ = err
+            raise exc
+
+    old_timeout = socket.getdefaulttimeout()
+    try:
+        if timeout is not None:
+            socket.setdefaulttimeout(timeout)
+        yield
+    except IOError as err:
+        # urllib can wrap original socket errors multiple times (!), we must
+        # unwrap to get at the original error.
+        while True:
+            a = err.args
+            if len(a) >= 1 and isinstance(a[0], IOError):
+                err = a[0]
+            # The error can also be wrapped as args[1]:
+            #    except socket.error as msg:
+            #        raise IOError('socket error', msg).with_traceback(sys.exc_info()[2])
+            elif len(a) >= 2 and isinstance(a[1], IOError):
+                err = a[1]
+            else:
+                break
+        filter_error(err)
+        raise
+    # XXX should we catch generic exceptions and look for their
+    # __cause__ or __context__?
+    finally:
+        socket.setdefaulttimeout(old_timeout)
 
 
 @contextlib.contextmanager
@@ -1236,6 +1293,8 @@ def disable_gc():
 
 def python_is_optimized():
     """Find if Python was built with optimizations."""
+    # We don't have sysconfig on Py2.6:
+    import sysconfig
     cflags = sysconfig.get_config_var('PY_CFLAGS') or ''
     final_opt = ""
     for opt in cflags.split():
@@ -1387,7 +1446,7 @@ class _MemoryWatchdog(object):
         try:
             f = open(self.procfile, 'r')
         except OSError as e:
-            warnings.warn('/proc not available for stats: {}'.format(e),
+            warnings.warn('/proc not available for stats: {0}'.format(e),
                           RuntimeWarning)
             sys.stderr.flush()
             return
@@ -1607,18 +1666,19 @@ def run_unittest(*classes):
     _filter_suite(suite, case_pred)
     _run_suite(suite)
 
-#=======================================================================
-# Check for the presence of docstrings.
-
-HAVE_DOCSTRINGS = (check_impl_detail(cpython=False) or
-                   sys.platform == 'win32' or
-                   sysconfig.get_config_var('WITH_DOC_STRINGS'))
-
-requires_docstrings = unittest.skipUnless(HAVE_DOCSTRINGS,
-                                          "test requires docstrings")
-
-
-#=======================================================================
+# We don't have sysconfig on Py2.6:
+# #=======================================================================
+# # Check for the presence of docstrings.
+# 
+# HAVE_DOCSTRINGS = (check_impl_detail(cpython=False) or
+#                    sys.platform == 'win32' or
+#                    sysconfig.get_config_var('WITH_DOC_STRINGS'))
+# 
+# requires_docstrings = unittest.skipUnless(HAVE_DOCSTRINGS,
+#                                           "test requires docstrings")
+# 
+# 
+# #=======================================================================
 # doctest driver.
 
 def run_doctest(module, verbosity=None, optionflags=0):
@@ -1657,7 +1717,12 @@ def modules_cleanup(oldmodules):
     # globals will be set to None which will trip up the cached functions.
     encodings = [(k, v) for k, v in sys.modules.items()
                  if k.startswith('encodings.')]
-    sys.modules.clear()
+    # Was:
+    # sys.modules.clear()
+    # Py2-compatible:
+    for i in range(len(sys.modules)):
+        sys.modules.pop()
+
     sys.modules.update(encodings)
     # XXX: This kind of problem can affect more than just encodings. In particular
     # extension modules (such as _ssl) don't cope with reloading properly.
@@ -1668,6 +1733,9 @@ def modules_cleanup(oldmodules):
     sys.modules.update(oldmodules)
 
 #=======================================================================
+# Backported versions of threading_setup() and threading_cleanup() which don't refer
+# to threading._dangling (not available on Py2.7).
+
 # Threading support to prevent reporting refleaks when running regrtest.py -R
 
 # NOTE: we use thread._count() rather than threading.enumerate() (or the
@@ -1680,20 +1748,20 @@ def modules_cleanup(oldmodules):
 
 def threading_setup():
     if _thread:
-        return _thread._count(), threading._dangling.copy()
+        return _thread._count(),
     else:
-        return 1, ()
+        return 1,
 
-def threading_cleanup(*original_values):
+def threading_cleanup(nb_threads):
     if not _thread:
         return
+
     _MAX_COUNT = 10
     for count in range(_MAX_COUNT):
-        values = _thread._count(), threading._dangling
-        if values == original_values:
+        n = _thread._count()
+        if n == nb_threads:
             break
         time.sleep(0.1)
-        gc_collect()
     # XXX print a warning in case of failure?
 
 def reap_threads(func):
