@@ -4,11 +4,8 @@ builtins into Py2. Has no effect on Py3.
 
 See the docs for these modules for more information::
 
-- future.builtins.types
+- future.types
 - future.builtins.iterators
-- future.builtins.newnext
-- future.builtins.newround
-- future.builtins.newsuper
 - future.builtins.misc
 - future.builtins.disabled
 
@@ -19,7 +16,26 @@ from future.builtins.iterators import (filter, map, zip)
 # backward-compatibility with future v0.8.2. It will be removed in future v1.0.
 from future.builtins.misc import (ascii, chr, hex, input, isinstance, next,
                                   oct, open, pow, round, super)
-from future.builtins.types import (bytes, dict, int, range, str)
+from future.utils import PY3
+
+if PY3:
+    import builtins
+    bytes = builtins.bytes
+    dict = builtins.dict
+    int = builtins.int
+    list = builtins.list
+    object = builtins.object
+    range = builtins.range
+    str = builtins.str
+    __all__ = []
+else:
+    from future.types import (newbytes as bytes,
+                              newdict as dict,
+                              newint as int,
+                              newlist as list,
+                              newobject as object,
+                              newrange as range,
+                              newstr as str)
 from future import utils
 
 
@@ -30,11 +46,10 @@ if not utils.PY3:
     # Only shadow builtins on Py2; no new names
     __all__ = ['filter', 'map', 'zip', 
                'ascii', 'chr', 'hex', 'input', 'next', 'oct', 'open', 'pow',
-               'round', 'super', 'bytes', 'dict', 'int', 'range', 'str',
+               'round', 'super',
+               'bytes', 'dict', 'int', 'list', 'object', 'range', 'str',
               ]
 
 else:
     # No namespace pollution on Py3
     __all__ = []
-
-    # TODO: add 'callable' for Py3.0 and Py3.1?
