@@ -4,7 +4,10 @@ Tests to make sure the behaviour of the builtins is sensible and correct.
 """
 
 from __future__ import absolute_import, division, print_function, unicode_literals
-from future.builtins import *
+from future.builtins import (bytes, dict, int, range, round, str, super,
+                             ascii, chr, hex, input, next, oct, open, pow,
+                             filter, map, zip)
+
 from future.utils import PY3, exec_, native_str, implements_iterator
 from future.tests.base import unittest, skip26
 
@@ -122,8 +125,6 @@ class TestBuiltins(unittest.TestCase):
 
         self.assertFalse(isinstance('blah', (bytes, Decimal, float, int)))
 
-    @unittest.skipIf(sys.version_info[:2] == (2, 6),
-                     'not yet implemented for Py2.6')
     def test_round(self):
         """
         Note that the Python 2.x round() function fails these tests. The
@@ -184,12 +185,9 @@ class TestBuiltins(unittest.TestCase):
 # Below here are the tests from Py3.3'2 test_builtin.py module
 ##############################################################
 
-from future import standard_library
-with standard_library.hooks():
-    import builtins
-    from test.support import TESTFN, unlink,  run_unittest, check_warnings
-    import ast
-    import collections
+from future.backports.test.support import TESTFN, unlink,  run_unittest, check_warnings
+import ast
+import collections
 
 import io
 import locale
@@ -201,7 +199,6 @@ import sys
 import traceback
 import types
 # Imported above more portably (using unittest2 on Py2.6):
-# import unittest
 import warnings
 from operator import neg
 try:
@@ -482,7 +479,7 @@ class BuiltinTest(unittest.TestCase):
         self.assertRaises(ValueError, chr, 0x00110000)
         self.assertRaises((OverflowError, ValueError), chr, 2**32)
 
-    @unittest.expectedFailure
+    @unittest.skip('FIXME: skip on narrow builds?')
     def test_ord_big(self):
         """
         These tests seem to fail on OS X (narrow Python build?)
@@ -499,8 +496,7 @@ class BuiltinTest(unittest.TestCase):
         self.assertEqual(ord("\U0010FFFE"), 0x0010FFFE)
         self.assertEqual(ord("\U0010FFFF"), 0x0010FFFF)
 
-
-    @unittest.expectedFailure
+    @unittest.skip('FIXME: skip on narrow builds?')
     def test_chr_big(self):
         """
         These tests seem to fail on OS X (narrow Python build?)
@@ -515,10 +511,6 @@ class BuiltinTest(unittest.TestCase):
         self.assertEqual(chr(0x0010FFFE), "\U0010FFFE")
         self.assertEqual(chr(0x0010FFFF), "\U0010FFFF")
 
-    # We disable this test, because __builtin__ becomes builtins on Py2
-    # def test_cmp(self):
-    #     self.assertTrue(not hasattr(builtins, "cmp"))
-
     def test_compile(self):
         compile('print(1)\n', '', 'exec')
         bom = b'\xef\xbb\xbf'
@@ -731,7 +723,11 @@ class BuiltinTest(unittest.TestCase):
 
         # Verify locals stores (used by list comps)
         eval('[locals() for i in (2,3)]', g, d)
-        eval('[locals() for i in (2,3)]', g, collections.UserDict())
+        if PY3:
+            from collections import UserDict
+        else:
+            from UserDict import UserDict
+        eval('[locals() for i in (2,3)]', g, UserDict())
 
         class SpreadSheet:
             "Sample application showing nested, calculated lookups."
