@@ -67,6 +67,7 @@ On Python 3, this decorator is a no-op.
 import types
 import sys
 import numbers
+import functools
 
 PY3 = sys.version_info[0] == 3
 PY2 = sys.version_info[0] == 2
@@ -159,29 +160,45 @@ if PY3:
 else:
     # Python 2
     def tobytes(s):
-        '''
-        Encodes to latin-1 (where the first 256 chars are the same as
-        ASCII.)
-        '''
         if isinstance(s, unicode):
             return s.encode('latin-1')
         else:
             return ''.join(s)
 
+tobytes.__doc__ = """
+    Encodes to latin-1 (where the first 256 chars are the same as
+    ASCII.)
+    """
+
 if PY3:
-    def native_str_to_bytes(s, encoding='ascii'):
+    def native_str_to_bytes(s, encoding='utf-8'):
         return s.encode(encoding)
 
-    def bytes_to_native_str(b, encoding='ascii'):
+    def bytes_to_native_str(b, encoding='utf-8'):
         return b.decode(encoding)
+
+    def text_to_native_str(t, encoding=None):
+        return t
 else:
     # Python 2
-    def native_str_to_bytes(s, encoding='ascii'):
-        return s
+    def native_str_to_bytes(s, encoding=None):
+        from future.types import newbytes    # to avoid a circular import
+        return newbytes(s)
 
-    def bytes_to_native_str(b, encoding='ascii'):
-        return b
+    def bytes_to_native_str(b, encoding=None):
+        return native(b)
 
+    def text_to_native_str(t, encoding='ascii'):
+        """
+        Use this to create a Py2 native string when "from __future__ import
+        unicode_literals" is in effect.
+        """
+        return unicode(t).encode(encoding)
+
+native_str_to_bytes.__doc__ = """
+    On Py3, returns an encoded string.
+    On Py2, returns a newbytes type, ignoring the ``encoding`` argument.
+    """
 
 if PY3:
     # list-producing versions of the major Python iterating functions
@@ -302,7 +319,7 @@ def bind_method(cls, name, func):
     -------
     None
     """
-    # only python 2 has bound/unbound method issue
+    # only python 2 has an issue with bound/unbound methods
     if not PY3:
         setattr(cls, name, types.MethodType(func, None, cls))
     else:
@@ -439,7 +456,7 @@ def isnewbytes(obj):
     """
     # TODO: generalize this so that it works with subclasses of newbytes
     # Import is here to avoid circular imports:
-    from future.builtins.types.newbytes import newbytes
+    from future.types.newbytes import newbytes
     return type(obj) == newbytes
 
 
@@ -519,11 +536,89 @@ def old_div(a, b):
     """
     Equivalent to ``a / b`` on Python 2 without ``from __future__ import
     division``.
+
+    TODO: generalize this to other objects (like arrays etc.)
     """
-    return a // b if (isint(a) and isint(b)) else a / b
+    if isinstance(a, numbers.Integral) and isinstance(b, numbers.Integral):
+        return a // b
+    else:
+        return a / b
+
+
+def as_native_str(encoding='utf-8'):
+    '''
+    A decorator to turn a function or method call that returns text, i.e.
+    unicode, into one that returns a native platform str.
+
+    Use it as a decorator like this::
+        
+        from __future__ import unicode_literals
+
+        class MyClass(object):
+            @as_native_str(encoding='ascii')
+            def __repr__(self):
+                return next(self._iter).upper()
+    '''
+    if PY3:
+        return lambda f: f
+    else:
+        def encoder(f):
+            @functools.wraps(f)
+            def wrapper(*args, **kwargs):
+                return f(*args, **kwargs).encode(encoding=encoding)
+            return wrapper
+        return encoder
+
+# listvalues and listitems definitions from Nick Coghlan's (withdrawn)
+# PEP 496:
+try:
+    dict.iteritems
+except AttributeError:
+    # Python 3
+    def listvalues(d):
+        return list(d.values())
+    def listitems(d):
+        return list(d.items())
+else:
+    # Python 2
+    def listvalues(d):
+        return d.values()
+    def listitems(d):
+        return d.items()
+
+if PY3:
+    def ensure_new_type(obj):
+        return obj
+else:
+    def ensure_new_type(obj):
+        from future.types.newbytes import newbytes
+        from future.types.newstr import newstr
+        from future.types.newint import newint
+        from future.types.newdict import newdict
+
+        native_type = type(native(obj))
+
+        # Upcast only if the type is already a native (non-future) type
+        if issubclass(native_type, type(obj)):
+            # Upcast
+            if native_type == str:  # i.e. Py2 8-bit str
+                return newbytes(obj)
+            elif native_type == unicode:
+                return newstr(obj)
+            elif native_type == int:
+                return newint(obj)
+            elif native_type == dict:
+                return newdict(obj)
+            else:
+                return NotImplementedError('type %s not supported' % type(obj))
+        else:
+            # Already a new type
+            assert type(obj) in [newbytes, newstr]
+            return obj
 
 
 __all__ = ['PY3', 'PY2', 'PYPY', 'python_2_unicode_compatible',
+           'as_native_str',
            'with_metaclass', 'bchr', 'bstr', 'bord',
            'tobytes', 'str_to_native_bytes', 'bytes_to_native_str', 
            'lrange', 'lmap', 'lzip', 'lfilter',
@@ -531,5 +626,7 @@ __all__ = ['PY3', 'PY2', 'PYPY', 'python_2_unicode_compatible',
            'viewitems', 'viewkeys', 'viewvalues',
            'bind_method', 'getexception',
            'reraise', 'implements_iterator', 'get_next', 'encode_filename',
-           'is_new_style', 'native_str']
+           'is_new_style', 'native_str', 'old_div', 'as_native_str',
+           'listvalues', 'listitems'
+          ]
 
